"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Definition = void 0;
const factories = require("./serviceFactories");
const randomName_1 = require("./randomName");
const TypeRef_1 = require("./TypeRef");
class Definition {
    constructor(name) {
        this.args = [];
        this.annotations = [];
        if (name) {
            this.name = name;
        }
        else {
            this.name = randomName_1.randomName();
        }
    }
    /**
     * Sets service constructor
     */
    useConstructor(constructor) {
        this.factory = factories.fromConstructor(constructor);
        this.type = TypeRef_1.TypeRef.createFromType(constructor);
        return this;
    }
    /**
     * Alias for {@see useConstructor}
     */
    useClass(clazz) {
        return this.useConstructor(clazz);
    }
    /**
     * Sets factory used to create an instance of service.
     * In case of asynchronous service creation, factory should return a promise.
     *
     * The factory value is called in context of AlphaDIC.
     */
    useFactory(factory, type) {
        this.factory = factories.fromFactory(factory);
        this.type = type && TypeRef_1.TypeRef.createFromType(type);
        return this;
    }
    /**
     * Sets information about type of final service
     */
    markType(type) {
        this.type = TypeRef_1.TypeRef.is(type) ? type : TypeRef_1.TypeRef.createFromType(type);
        return this;
    }
    /**
     * Defined service as provided value
     */
    useValue(value) {
        this.factory = factories.fromValue(value);
        this.type = TypeRef_1.TypeRef.createFromValue(value);
        return this;
    }
    /**
     * Sets the array of arguments provided to service factory.
     * All arguments are provided directly to service constructor or
     * factory unless they are an instance of ContainerArg which has to be resolved first
     */
    withArgs(...args) {
        this.args = args;
        return this;
    }
    /**
     * Adds annotation to the service
     * Annotation is a simple metadata object assigned to service that you might use for different purposes
     */
    annotate(...annotations) {
        this.annotations.push(...annotations);
        return this;
    }
    static create(name) {
        return new Definition(name);
    }
    /**
     * Locks definitions by making it immutable
     */
    lock() {
        Object.freeze(this);
        Object.freeze(this.args);
        Object.freeze(this.annotations);
        return this;
    }
    /**
     * Returns new, locked one, object of Definition with new data
     */
    modify(data) {
        const def = new Definition(this.name);
        Object.assign(def, this, data);
        return def.lock();
    }
}
exports.Definition = Definition;
